﻿// JavaScript file for https://www.ikz-berlin.de/~bickermann/[e-][papers,talks,confs].html
// (c) 2021 P. and M. Bickermann
// v1.1 (Feb 25, 2021) - remove SetStylesheet, remove .selected in updateSelector
// v1.2 (Oct 24, 2023) - change box.sorting class to new search element

// function, looks if an <li> element is a header
isHeader = elem => elem.children[0].nodeName == "H1";

// if URL starts with "e-" it's the english version
isEnglish = () => document.location.toString().substring(document.location.toString().lastIndexOf("/") + 1).startsWith("e-");

// onload: add event listeners
document.addEventListener("DOMContentLoaded", event => {

    // 1. append link to top of page to every h1
    var localizedTitle = isEnglish() ? "Top of Page" : "Zum Seitenanfang";
    var topLink = "<a href='#' title='" + localizedTitle + "'></a>";
    // append this to every h1 element in the ul.sortlist li
    // (for noscript, the h1 are 'display: none' anyway)
    document.querySelectorAll(".sortlist li").forEach(currElement => {
        // auto-generate id values from data-name
        if (currElement.getAttribute("data-name") != "na") { currElement.id = currElement.getAttribute("data-name"); }
        if (isHeader(currElement)) {
            // add class.h1 to li parent
            currElement.classList.add("h1");
            // append top link to h1
            currElement.children[0].innerHTML += topLink;
        }
    });

    // 2. evaluate URL GET-Parameters, that are the sort parameters for the function
    // the first line tests if the parameter exists and provides a default
    var sortType = new Parameters()["sort"] || "date";
    // the second line confines to valid parameter names (resp. bool output)
    if (!["date", "name", "publ", "area"].includes(sortType)) { sortType = "date"; }
    var reverse = new Parameters()["reverse"] || "true";
    reverse = reverse == "true";

    // 3. Sort the ul.sortlist
    // (and assigns id to every visible list entry)
    sortList(sortType, reverse);

    // 4. Update the search selector
    // (and highlights the selected fragment if available)
    updateSelector(sortType, reverse);

    // sortlist click event
    document.querySelectorAll("search a").forEach(currValue => {
        currValue.addEventListener("click", event => {
            // 1. Sort the ul.sortlist
            sortList(event.target.getAttribute("data-sort"), event.target.hasAttribute("reverse"));
            // 2. Update the search selector
            updateSelector(event.target.getAttribute("data-sort"), event.target.hasAttribute("reverse"));
        });
    });

     // interne links zu den Manuskript-PDFs anzeigen
    document.getElementById("super") && document.getElementById("super").addEventListener("click", event => {
        document.querySelectorAll(".tref, .pref").forEach(curr => {
            curr.style.display = "inline";
        });
    });
});

// Update the search selector
function updateSelector(sortType, reverse) {
    // box is .sorting, indicator is .sort, attribute is data-sort; reverse is not evaluated
    // delete all selector (including the old one)
    document.querySelectorAll("search .sort").forEach(curr => { curr.classList.remove("sort"); });
    // add new selector if it exists
    document.querySelector("search a[data-sort='" + sortType + "']") && document.querySelector("search a[data-sort='" + sortType + "']").classList.add("sort");
    // recreate URL path to display
    var url = document.location.toString();
    var fragment = url.indexOf("#") > 0 ? url.substring(url.indexOf("#"), url.length) : "";
    var filename = url.substring(0, url.indexOf("?"));
    // change URL in browser without reload (also fragment is displayed)
    window.history.pushState("", "", filename += "?sort=" + sortType + "&reverse=" + reverse + fragment);
}

// sort a list and display headers according to data-sort attribute

// 1st parameter: data-sort attribute (without 'data-') as string (e.g. 'date', 'name', 'publ' or 'area')
// 2nd parameter: reverse (sorting order) as boolean (Z...A instead of A...Z)
// for each (1) <aside> and (2) <li> in <ul.sortlist>:
// - everything with data-* = 'na' will get 'display: none'
// - every other value will get 'display: block'
// - the attribute itself MUST be present on each <aside> and <li>!

function sortList(sortType, reverse) {
    // remove all list entries where the data-* attribute is 'na' (not available)
    document.querySelectorAll("li[data-" + sortType + "=na], aside[data-" + sortType + "=na]").forEach(curr => { curr.style.display = "none"; });
    // and bring back those where the data-* attribute matches (e.g. if previously hidden)
    document.querySelectorAll("li[data-" + sortType + "]:not([data-" + sortType + "=na])").forEach(curr => { curr.style.display = "list-item"; });
    document.querySelectorAll("aside:not([data-" + sortType + "=na])").forEach(curr => { curr.style.display = "block"; });

    var invisibleNodes = [];
    // get the list from the DOM and filter out invisible elements
    var publs = Array.prototype.slice.call(document.querySelectorAll(".sortlist li")).filter(curr => {
        if (curr.style.display == "none") {
            // save it for later
            invisibleNodes.push(curr);
            return false;
        } else {
            return true;
        }
    });
    // sort the list (and every block)
    publs.sort(sortFunction(sortType, reverse));
    publs = sortBlocks(publs, sortType);
    // append hidden nodes to the end
    publs = publs.concat(invisibleNodes);
    // each item is appended to the DOM
    publs.forEach(currValue => { currValue.parentNode.appendChild(currValue); });
    // while the publs is used to generate statistics (see below)
    countPubls(publs, sortType);
}

// function to use with Array.sort()
function sortFunction(sortType, reverse) {
    return (a, b) => {
        if (reverse) {
            // if it's reverse, simply switch a & b
            [a, b] = [b, a];
        }
        return a.getAttribute("data-" + sortType).localeCompare(b.getAttribute("data-" + sortType));
    };
}

// sorts every block separately by date
function sortBlocks(publs, sortType) {
    var output = [];
    publs.forEach(curr => {
        if (isHeader(curr)) {
            var headerData = curr.getAttribute("data-" + sortType);
            var tmp = [];
            publs.forEach(val => {
                if (!isHeader(val) && val.getAttribute("data-" + sortType).startsWith(headerData)) {
                    tmp.push(val);
                }
            });
            tmp.sort(sortFunction("date", true)).unshift(curr);
            output = output.concat(tmp);
        }
    });
    return output;
}

// count publications: list statistics as Array (see debug output)
function countPubls(publs, sortType) {
    var counts = [["", 0]];
    var currIndex = 0;
    publs.forEach(val => {
        // don't consider hidden elements
        if (val.style.display == "none") {return;}
        if (isHeader(val)) {
        // if it's a visible header
            currIndex++;
            // the next item count starts
            counts.push([val.innerText, 0]);
        } else {
            // count the current item abundance
            counts[currIndex][1]++;
        }
    });
    // remove the first (always empty) element
    counts.shift();
    // debug output for future use
    console.debug(counts);
    // another debug output to check if all entries are actually there...
    var entries = 0, headers = 0, invisible = 0;
    counts.forEach(curr => {entries+=curr[1];});
    publs.forEach(curr => {headers+=isHeader(curr);});
    publs.forEach(curr => {invisible+=curr.style.display == "none";});
    console.debug("entries: " + entries + ", headers: " + (headers-invisible) + ", hidden:" + invisible + ", total: "+ publs.length);
}

// return an object with all GET parameters from the URL
// copied from: https://wiki.selfhtml.org/wiki/JavaScript/Tutorials/Wert%C3%BCbergabe_zwischen_verschiedenen_HTML-Dokumenten#Wert.C3.BCbergabe_mittels_URL
function Parameters() {
    if (location.search == '') return;
    var pairs = location.search.slice(1).split("&");
    var key, value;
    pairs.forEach(curr => {
        key = curr.split("=")[0];
        value = curr.split("=")[1];
        this[key] = value;
    });
}
